@@ -21,6 +21,8 @@ import string
 from kfp.dsl import ContainerOp
 from kfp_notebook import __version__
 from kubernetes.client.models import V1EmptyDirVolumeSource, V1EnvVar, V1Volume, V1VolumeMount
+from kubernetes.client.models import V1EnvVarSource
+from kubernetes.client.models import V1ObjectFieldSelector
 from typing import Dict, List, Optional
 
 
@@ -71,6 +73,7 @@ class NotebookOp(ContainerOp):
                  cpu_request: Optional[str] = None,
                  mem_request: Optional[str] = None,
                  gpu_limit: Optional[str] = None,
+                 workflow_engine: Optional[str] = 'argo',
                  **kwargs):
         """Create a new instance of ContainerOp.
         Args:
@@ -92,6 +95,7 @@ class NotebookOp(ContainerOp):
           cpu_request: number of CPUs requested for the operation
           mem_request: memory requested for the operation (in Gi)
           gpu_limit: maximum number of GPUs allowed for the operation
+          workflow_engine: Kubeflow workflow engine, defaults to 'argo'
           kwargs: additional key value pairs to pass e.g. name, image, sidecars & is_exit_handler.
                   See Kubeflow pipelines ContainerOp definition for more parameters or how to use
                   https://kubeflow-pipelines.readthedocs.io/en/latest/source/kfp.dsl.html#kfp.dsl.ContainerOp
@@ -241,6 +245,20 @@ class NotebookOp(ContainerOp):
             gpu_vendor = self.pipeline_envs.get('GPU_VENDOR', 'nvidia')
             self.container.set_gpu_limit(gpu=str(gpu_limit), vendor=gpu_vendor)
 
+        # Generate unique ELYRA_RUN_NAME value and expose it as an environment
+        # variable in the container
+        if workflow_engine and workflow_engine.lower() == 'argo':
+            run_name_placeholder = '{{workflow.annotations.pipelines.kubeflow.org/run_name}}'
+            self.container.add_env_variable(V1EnvVar(name='ELYRA_RUN_NAME',
+                                                     value=run_name_placeholder))
+        else:
+            # For Tekton derive the value from the specified pod annotation
+            annotation = 'pipelines.kubeflow.org/run_name'
+            field_path = f"metadata.annotations['{annotation}']"
+            self.container.add_env_variable(V1EnvVar(name='ELYRA_RUN_NAME',
+                                                     value_from=V1EnvVarSource(
+                                                         field_ref=V1ObjectFieldSelector(field_path=field_path))))
+
         # Attach metadata to the pod
         # Node type (a static type for this op)
         self.add_pod_label('elyra/node-type',
